#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# get traveltime residuals using synthetic 1D travel times from FMTOMO hybrid method
#
# this script is used to make sure that synthetic travel times are the same for synthetic data residuals (rtimes)
# and observed data residuals, because of small differences between hybrid 1D solver and taup method
# Therefore, correlation output times are absolute (minus source time) values and residuals are calcuated using
# FMTOMO hybrid reference times (e.g. rtimes.dat)
#
import os
import argparse
import shutil
import numpy as np
import matplotlib.pyplot as plt

from pylot.tomography.fmtomo_tools.fmtomo_teleseismic_utils import organize_event_names

# Note: weighting of mean might not be reasonable due to uncertainty distribution along the array
def main(infile_rtimes, fn_events, kde_file, demean=True, no_deref=False, weight_kde=True, weight=False):
    eventnames = organize_event_names(fn_events)
    assert not(weight and weight_kde), 'Cannot use two weights'

    if not no_deref:
        reftimes = np.genfromtxt(infile_rtimes)
        if weight_kde:
            kde_values = np.genfromtxt(kde_file)
            assert len(kde_values) == len(reftimes), 'Missmatch in reftimes and kde file length'

    nSrc = len(eventnames)

    diff_in_means = []
    # iterate over all sources
    for index in range(nSrc):
        srcid = index + 1
        eventname = eventnames[index]
        print('{} ({}/{})'.format(eventname, srcid, nSrc))
        # get pick filename and read observed times
        pickfile = os.path.join('picks', eventname)
        obsdata_src = np.loadtxt(pickfile, skiprows=3)
        # make safety copy
        picksafedir = os.path.join('picks', 'save')
        if not os.path.isdir(picksafedir):
            os.mkdir(picksafedir)
        shutil.copy(pickfile, picksafedir)
        npicks = len(obsdata_src)
        # read observed times from pickfile
        tt_obs = obsdata_src[:, 3]
        # read uncertainties from pickfile
        uncs = obsdata_src[:, 4]

        if no_deref:
            tt_res = tt_obs
        else:
            indices = np.where(reftimes[:, 1] == srcid)[0]
            assert (len(indices) == npicks), 'Missmatch in indices for srcids'
            # read reference teleseismic times from FMTOMO run
            tt_ref = reftimes[:, 4][indices]
            # calculate residuals
            tt_res = tt_obs - tt_ref

        # set new residual to obsdata array
        obsdata_src[:, 3] = tt_res
        # get kde values
        #weights = 1./kde_values[:, 2][indices] if weight_kde else None
        #
        weights = 1. / uncs ** 2 if weight else None
        # demean residuals for current source
        mean = np.average(tt_res, weights=weights)
        mean_unweighted = np.mean(tt_res)
        diff_in_means.append(mean - mean_unweighted)
        print('Mean:', mean)
        print('Unweighted Mean:', mean_unweighted)
        print('Demean setting:', demean)
        if demean:
            obsdata_src[:, 3] -=  mean
        #obsdata_src[:, 3] = mtimes[:, 4][indices]
        # Write new pickfiles
        with open(pickfile, 'r') as infile_pick:
            header = infile_pick.readlines()[:3]
        with open(pickfile, 'w') as outfile_pick:
            for line in header:
                outfile_pick.write(line)
            for line in obsdata_src:
                outfile_pick.write('{:10.4f} {:10.4f} {:10.4f} {:15.8f} {:15.8f}\n'.format(*line))
    plt.hist(diff_in_means, bins=100)
    plt.title('Diffs in means (weighted - unweighted) [s]')
    plt.show()


if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        description='Calculate residuals for absolute observation times using reference file from FMTOMO')
    parser.add_argument('--rtimes', default='rtimes_tele.dat', help='reference_file')
    parser.add_argument('--sourcefile', default='input_source_file_P.in', help='input_source_file')
    #parser.add_argument('--kdefile', default='kde_weights.txt', help='input file with station kde values for weighting')
    parser.add_argument('-nd', '--no_demean', action='store_true', default=False,
                        help='do not mean correct travel times')
    parser.add_argument('-nr', '--no_reftimes', action='store_true', default=False,
                        help='do not use reference times (e.g. picks are already tt residuals). rtimes file still required at the moment.')
    #parser.add_argument('-nwk', '--no_weight_kde', action='store_true', default=True, help='do not weight travel times')
    args = parser.parse_args()
    #args.kdefile
    main(args.rtimes, args.sourcefile, None, not(args.no_demean), weight_kde=False, no_deref=args.no_reftimes) #not(args.no_weight))