import os
import json

import numpy as np
import matplotlib.pyplot as plt

from pylot.tomography.fmtomo_tools.fmtomo_teleseismic_utils import organize_receivers

# TODO: demeaning source by source??

def calc_misfit(data, synth, unc):
    return np.sum(((data - synth) / unc) ** 2) / len(data)

def calc_mean_residuals(data, synth):
    return np.sum(abs(data - synth)) / len(data)


def sort_stations_for_search(station_coords, receivers_dict):
    epsilon = 1e-1 # very large epsilon actually...
    coords_unique = coords_unique_receivers(receivers_dict)
    latlon_dict = {}
    for nwst_id, coords in station_coords.items():
        lat = coords['latitude']
        lon = coords['longitude']
        for latu, lonu in coords_unique:
            if abs(lat - latu) < epsilon and abs(lon - lonu) < epsilon:
                latlon_dict[(latu, lonu)] = nwst_id
    return latlon_dict


def extract_stations_for_network(station_coords, network_id):
    station_coords_extracted = {}
    for nwst_id, coords in station_coords.items():
        nw, st = nwst_id.split('.')
        if nw == network_id:
            station_coords_extracted[nwst_id] = coords
    return station_coords_extracted


def get_receiver_ids(latlon_list, receivers_dict):
    receiver_ids = []
    for recid, coords in receivers_dict.items():
        lat, lon = (coords['lat'], coords['lon'])
        if (lat, lon) in latlon_list:
            receiver_ids.append(recid)
    return receiver_ids


def coords_unique_receivers(receivers_dict):
    coords_unique = []
    for coords in receivers_dict.values():
        latlon_tuple = (coords['lat'], coords['lon'])
        if not latlon_tuple in coords_unique:
            coords_unique.append(latlon_tuple)
    return coords_unique

#def plot_residuals()

def main():
    #wdir = '/rscratch/minos13/marcel/fmtomo_alparray/alparray_mantle_diehl_crust_included_v3_hf'
    wdir = '/data/AlpArray_Data/fmtomo/v5/crust_incl_hf_sm_FIX_DTS_grad_sm30_dm10'
    os.chdir(wdir)

    receivers_dict = organize_receivers('receivers.in')

    print(wdir)
    misfits = {}
    mean_residuals = {}
    for extract_network in [None, 'Z3', 'ZS', ]:
        misfits[extract_network] = []
        mean_residuals[extract_network] = []
        with open('/rscratch/minos13/marcel/alparray/station_coords.json', 'r') as infile:
            station_coords = json.load(infile)

        print('\nCalculating residuals for network:', extract_network)
        if extract_network:
            station_coords = extract_stations_for_network(station_coords, extract_network)
        latlon_dict = sort_stations_for_search(station_coords, receivers_dict)
        rec_ids = get_receiver_ids(list(latlon_dict.keys()), receivers_dict)

        otimes = np.genfromtxt('otimes.dat', skip_header=1)
        #otimes_orig = np.genfromtxt('otimes_orig.dat', skip_header=1)
        ttimes = otimes[:, 4]
        uncs = otimes[:, 5]
        nsrc = int(otimes[-1, 1])

        rtimes = np.genfromtxt('rtimes_tele.dat')[:, 4]

        for itstep in range(0, 25):
            nRays = 0
            mtimes_all = np.genfromtxt('it_{}/arrivals.dat'.format(itstep))
            mtimes = mtimes_all[:, 4]
            mtimes_diff = mtimes - rtimes

            for srcid in range(nsrc):
                srcid += 1
                # get all indices of current source id
                indices = np.where(mtimes_all[:, 1] == srcid)
                nRays += len(indices[0])
                mtimes_diff[indices] -= np.mean(mtimes_diff[indices])

            # get all indices that are in rec_ids list as well
            mask_recs = np.isin(mtimes_all[:, 0].astype(int), rec_ids)

            mf = calc_misfit(ttimes[mask_recs], mtimes_diff[mask_recs], uncs[mask_recs])
            sr = calc_mean_residuals(ttimes[mask_recs], mtimes_diff[mask_recs])
            misfits[extract_network].append(mf)
            mean_residuals[extract_network].append(sr)
            print('It: {}, misfit: {}, mean res: {}, nrays: {}'.format(itstep, mf, sr, nRays))

        #plt.plot(mean_residuals[extract_network], label=extract_network)
        plt.plot(misfits[extract_network], label=extract_network)

    plt.title(wdir)
    plt.ylabel('Mean residuals [s]')
    plt.xlabel('Iteration')
    plt.legend()
    plt.show()

if __name__ == '__main__':
    main()





