#!/usr/bin/env python
# -*- coding: utf-8 -*-

import glob
import os
import json
import numpy as np

import matplotlib.pyplot as plt

import cartopy
import cartopy.crs as ccrs
from cartopy.io.shapereader import Reader
from matplotlib.patches import Patch

#from cmcrameri import cm

from obspy.geodetics import gps2dist_azimuth

TRANSFORM = ccrs.PlateCarree()


def draw_schmid_faults(ax, fnin='schmidfaults.shp'):
    reader = Reader(fnin)
    lines = []
    linestyles = []
    for record in reader.records():
        info = record.attributes
        if info["fault_type"] == 1 or info["fault_type"] == 3:
            linestyles.append('solid')
        else:
            linestyles.append('dashed')
        line_arr = np.array(record.geometry.coords)
        lines.append(line_arr)
    # ax.add_collection(LineCollection(lines, linewidths=1.2, linestyles=linemarkers, colors='black', transform=ccrs.PlateCarree()))
    for line, style in zip(lines, linestyles):
        ax.plot(line[:, 0], line[:, 1], transform=TRANSFORM, ls=style, c='k', lw=1.2)


def draw_alcapadi_faults(ax, fnin='faults_alcapadi', color='black'):
    reader = Reader(fnin)
    lines = []
    linestyles = []
    linewidths = []
    for record in reader.records():
        info = record.attributes
        if info["fault_type"] == 1:
            linestyles.append('solid')
            linewidths.append(.8)
        elif info["fault_type"] == 2:
            linestyles.append('solid')
            linewidths.append(0.4)
        else:
            linestyles.append('dashed')
            linewidths.append(.8)
        line_arr = np.array(record.geometry.coords)
        lines.append(line_arr)
    for line, style, lwidth in zip(lines, linestyles, linewidths):
        ax.plot(line[:, 0], line[:, 1], transform=TRANSFORM, ls=style, c='k', lw=lwidth)


def draw_alcapadi_model(ax, fnin='tect_units_alcapadi', alpha=0.2, add_legend=True):
    reader = Reader(fnin)
    color_dict = {"Adria accreted": (0.8, 0.7, 0.45, alpha),
                  "Adria autochton": (0.52, 0.32, 0.2, alpha),
                  "Europe accreted": (0.42, 0.67, 0.88, alpha),
                  "Flexural foredeep and graben fill": (0.6, 0.6, 0.6, alpha),  # (1.0, 1.0, 220/255., alpha),
                  "Alpine Tethys": (0., 0.65, 0.3, alpha),
                  "Neotethys": (0.5, 0.8, 0.32, alpha)}

    patches_legend = [Patch(color=value, label=key.capitalize()) for key, value in color_dict.items()]

    for record in reader.records():
        info = record.attributes
        shape = record.geometry
        color = color_dict.get(info['tect_unit'])
        if not color:
            color = (1.0, 1.0, 1.0, alpha)
        ax.add_geometries(shape, crs=TRANSFORM, facecolor=color)

    if add_legend:
        ax.legend(handles=patches_legend, ncol=3, bbox_to_anchor=(0.5, -0.075), loc='center')


def init_cartopy_map(fig=None, draw_mapbound=True, fill_continents=False,
                     continents_color=None, mapbound_color=None, lakes_color=None, clon=None, clat=None):
    if not fig:
        fig = plt.figure()
    #projection = ccrs.LambertConformal(central_longitude=clon, central_latitude=clat)
    projection = ccrs.PlateCarree(central_longitude=clon)
    ax = fig.add_subplot(111, projection=projection)

    if fill_continents:
        ax.add_feature(cartopy.feature.LAND, color=continents_color)
        ax.add_feature(cartopy.feature.LAKES, color=lakes_color)
    if draw_mapbound:
        #ax.add_feature(cartopy.feature.OCEAN, color='w', linewidth=0.1)
        ax.add_feature(cartopy.feature.BORDERS, linewidth=0.2, color='0.3')
        ax.add_feature(cartopy.feature.COASTLINE, linewidth=0.3, color='0.3')
    return ax


def make_map(fig=None, draw_model=False, model_legends=True, draw_faults=False, width=20,
             height=14, clon=11, clat=46., draw_grid=True, continents='0.8', lakes='0.85', no_content=False,
             no_fill=True, alpha_model=0.2, faults_color='k', station_file=None):

    ax = init_cartopy_map(fig=fig, draw_mapbound=not no_content, fill_continents=not no_fill,
                          continents_color=continents, lakes_color=lakes, clon=clon, clat=clat)

    if station_file:
        with open(station_file, 'r') as fid:
            stations = json.load(fid)

        lons, lats = zip(*[(sta['longitude'], sta['latitude']) for sta in stations.values()])
        ax.scatter(lons, lats, c='0.3', s=1, transform=ccrs.PlateCarree(), zorder=5, edgecolors='none', alpha=0.5)

    # if draw_topo:
    #     draw_topo_model(basemap)
    if draw_model:
        fnin = '/home/marcel/sciebo/AlpArray_home/tectonic_maps_4dmb_2020_09_17/shape_files/tect_units_alcapadi'
        draw_alcapadi_model(ax, fnin, add_legend=model_legends, alpha=alpha_model)
    if draw_faults:
        fnin = '/home/marcel/sciebo/AlpArray_home/tectonic_maps_4dmb_2020_09_17/shape_files/faults_alcapadi'
        draw_alcapadi_faults(ax, fnin, faults_color)

    # if not no_content:
    #     basemap.drawcountries(color=line_color, linewidth=0.2)
    #     basemap.drawcoastlines(color=line_color, linewidth=0.3)

    if draw_grid:
        gl = ax.gridlines(crs=TRANSFORM, draw_labels=True,
                          linewidth=0.5, color='gray', alpha=0.5, linestyle=':')
        # dashes = [3, 6]
        # parallels = list(np.arange(-90, 90, lgrid))
        # parallels_small = [item for item in np.arange(-90, 90, sgrid) if not item in parallels]
        # basemap.drawparallels(parallels_small, dashes=dashes, color=line_color, linewidth=0.1, zorder=7)
        # basemap.drawparallels(parallels, dashes=[], color=line_color, linewidth=0.2, zorder=7, labels=[1, 1, 0, 0])
        # meridians = list(np.arange(-180, 180, lgrid))
        # meridians_small = [item for item in np.arange(-180, 180, sgrid) if not item in meridians]
        # basemap.drawmeridians(meridians_small, dashes=dashes, color=line_color, linewidth=0.1, zorder=7)
        # basemap.drawmeridians(meridians, dashes=[], color=line_color, linewidth=0.2, zorder=7, labels=[0, 0, 1, 1])

    ax.set_extent([clon - width/2, clon + width/2, clat - height/2, clat + height/2])

    return ax


def angle_marker(a1, a2, delta=1.):
    a1 = np.deg2rad(a1)
    a2 = np.deg2rad(a2)
    delta = np.deg2rad(delta)
    x_vals = [np.sin(angle) for angle in np.arange(a1, a2 + delta, delta)]
    y_vals = [np.cos(angle) for angle in np.arange(a1, a2 + delta, delta)]
    xy = zip(x_vals, y_vals)
    #x1 = np.sin(a1)
    #y1 = np.cos(a1)
    #x2 = np.sin(a2)
    #y2 = np.cos(a2)
    marker = [(0, 0), *xy, (0, 0)]
    return marker